/*
** mktty.c                      Generate the /etc/ttys, /etc/ttytype and
**                              /etc/securetty files from the /etc/ttytab file
**                              (for 4.3BSD compatibility).
**
** Version: 1.1
**
** Author: Peter Eriksson, Lysator Computer Club (pen@lysator.liu.se)
*/

#include <stdio.h>
#include <ttyent.h>
#include "gettytab.h"
  

static char *ttys_s     = "/etc/ttys";
static char *ttytype_s  = "/etc/ttytype";
static char *sectty_s   = "/etc/securetty";


main(argc, argv)
  int argc;
  char *argv[];
{
  FILE *ttys, *ttytype, *sectty;
  struct ttyent *ttp;
  char cmd[50], cfg[50];
  int flag;
  char cfgc;

  if (argc > 1)
    ttys_s = argv[1];
  if (argc > 2)
    ttytype_s = argv[2];
  if (argc > 3)
    sectty_s = argv[3];
  
  ttys	  = fopen(ttys_s, "w");
  ttytype = fopen(ttytype_s, "w");
  sectty  = fopen(sectty_s, "w");
  
  if (!ttys || !ttytype || !sectty)
  {
    fprintf(stderr, "could not open files\n");
    exit(1);
  }

  setttyent();

  while (ttp = getttyent())
  {
    cfgc = '2';
    
    fprintf(ttytype, "%s%s\t%s\n", ttp->ty_type,
	    (strlen(ttp->ty_type) < 8 ? "\t" : "" ), ttp->ty_name);

    if (ttp->ty_status & TTY_SECURE)
      fprintf(sectty, "%s\n", ttp->ty_name);
    
    if ((ttp->ty_status & TTY_ON) && ttp->ty_getty)
    {
      cmd[0] = cfg[0] = '\0';
      sscanf(ttp->ty_getty, "%s %s", cmd, cfg);
      if (strcmp(cmd, "/sbin/getty") || !cfg[0])
	flag = 0;
      else
      {
	cfgc = get_gettyentry(cfg);
	if (cfgc == -1)
	{
	  cfgc = get_gettyentry("std.9600");
	  fprintf(stderr, "illegal entry (unknown gettytab entry): %s\n",
		  ttp->ty_name);
	  flag = 0;
	}
	else
	  flag = 1;
      }
    }
    else
      flag = 0;
    
    fprintf(ttys, "%d%c%s\n", flag, cfgc, ttp->ty_name);
  }
}



get_gettyentry(name)
  char *name;
{
  static char buf[8000];


  if (getent(buf, name) < 0)
    return -1;
  else
    return buf[0];
}
